
/*
Mesh_Generator uses well known height map techniques and algorythms, these are widely used in the games industry to produce terrains for levels in games, the innovative twist is to consider the features of a face as topology, therefore , meshes of faces can be produced using these procedures.

this procedure has some limitations as the produced mesh is semi-open, that is, is not a closed geometry
solid ( as a head would be) so in order to reproduce a head using this method, 4 photographs had to be used, orthogonaly taked from front,side_1,side_2,back, the 4 produced meshes will then be joined.

The procedure presented here will produce the mesh for a face only.
ImageLoader.c
Credit is given to http://nehe.gamedev.net, http://www.gametutorials.com/, from where inspiration was drawn, and a couple of code snippets too. The whole rest and the  OpenGL rendering context was written by me with a little help of Rob (the bloke).

*/


#include<stdio.h>
#include<stdlib.h>
#include<GL/gl.h>
#include<GL/glu.h>
#include<GL/glut.h>
#include<string.h>
#include<math.h>
#include"ImageLoader.h"




/* Loads The .RAW File And Stores It In pHeightMap */
void LoadRawFile(const char* strName, int nSize,unsigned char *pHeightMap)
{
	int result;
	FILE *pFile = NULL;

	/* Open The File In Read / Binary Mode. */
	pFile = fopen( strName, "rb" );

	/* Check To See If We Found The File And Could Open It */
	if ( pFile == NULL )
	{
		/* Display Error Message And Stop The Function */
		printf("Error can't Find The Height Map!\n");
		return;
	}

	fread( pHeightMap, 1, nSize, pFile );

	/* Check If Everything Read Fine */
	result = ferror( pFile );

	/* Check If We Received An Error */
	if (result)
	{
		printf("Error failed To Get Data!\n");
	}
	else
		printf("file loaded OK!\n");

	/* Close The File. */
	fclose(pFile);
}

int Height(unsigned char *pHeightMap, int X, int Y) /* This Returns The Height From A Height Map Index */
{
	int x = X % MAP_SIZE;								/* Error Check Our x Value */
	int y = Y % MAP_SIZE;								/* Error Check Our y Value */

	if(!pHeightMap) return 0;							/* Make Sure Our Data Is Valid */

	return pHeightMap[x + (y * MAP_SIZE)];				/* Index Into Our Height Array And Return The Height */
}

void SetVertexColor(unsigned char *pHeightMap, int x, int y)	/* Sets The Color Value For A Particular Index, Depending On The Height Index, Nice Gradient  */
{
	if(!pHeightMap)
		return;								/* Make Sure Our Height Data Is Valid */
	else
	{
		float fColor = -0.15f + (Height(pHeightMap, x, y ) / 256.0f);
		/* Assign This Blue Shade To The Current Vertex */
		glColor3f(0, 0, fColor );

	}
}

void RenderHeightMap(unsigned char pHeightMap[])		/* This Renders The Height Map As Quads */
{
	int X = 0, Y = 0;									/* Create Some Variables To Walk The Array With. */
	int x, y, z;										/* Create Some Variables For Readability */

	if(!pHeightMap) return;							/* Make Sure Our Height Data Is Valid */

	if(filling)											
		glBegin( GL_QUADS );							
	else
		glBegin( GL_LINES );							

	for ( X = 0; X < MAP_SIZE; X += STEP_SIZE )
		for ( Y = 0; Y < MAP_SIZE; Y += STEP_SIZE )
		{
			/* Get The (X, Y, Z) Value For The Bottom Left Vertex */
			x = X;							
			y = Height(pHeightMap, X, Y );
			z = Y;

			/* Set The Color Value Of The Current Vertex */
			SetVertexColor(pHeightMap, x, z);

			glVertex3i(x, y, z);						/* Send This Vertex To OpenGL To Be Rendered (Integer Points Are Faster) */

			/* Get The (X, Y, Z) Value For The Top Left Vertex */
			x = X;										
			y = Height(pHeightMap, X, Y + STEP_SIZE );
			z = Y + STEP_SIZE ;

			/* Set The Color Value Of The Current Vertex */
			SetVertexColor(pHeightMap, x, z);

			glVertex3i(x, y, z);						/* Send This Vertex To OpenGL To Be Rendered */

			/* Get The (X, Y, Z) Value For The Top Right Vertex */
			x = X + STEP_SIZE;
			y = Height(pHeightMap, X + STEP_SIZE, Y + STEP_SIZE );
			z = Y + STEP_SIZE ;

			/* Set The Color Value Of The Current Vertex */
			SetVertexColor(pHeightMap, x, z);
			
			glVertex3i(x, y, z);						/* Send This Vertex To OpenGL To Be Rendered */

			/* Get The (X, Y, Z) Value For The Bottom Right Vertex */
			x = X + STEP_SIZE;
			y = Height(pHeightMap, X + STEP_SIZE, Y ); 
			z = Y;

			/* Set The Color Value Of The Current Vertex */
			SetVertexColor(pHeightMap, x, z);

			glVertex3i(x, y, z);						/* Send This Vertex To OpenGL To Be Rendered */
		}
	glEnd();

	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);					/*Reset The Color */
}


int drawGLScene(GLvoid)
{

      //gluLookAt(212, 60, 194,  186, 55, 171,  0, 1, 0);
	glScalef(scaleValue, scaleValue * HEIGHT_RATIO, scaleValue);

	RenderHeightMap(g_HeightMap);						

	return filling;										
}



